@@ -4,45 +4,42 @@
 #include <KLocalizedString>
 #include <QBitArray>
 #include <QJSValue>
-#include "incidencewrapper.h"
+#include <incidencewrapper.h>
+#include <QDebug>
 
 IncidenceWrapper::IncidenceWrapper(QObject *parent)
-    : QObject(parent)
-    , Akonadi::ItemMonitor()
+: QObject(parent)
+, Akonadi::ItemMonitor()
 {
-    // Change incidence pointer in remindersmodel if changed here
-    // connect(this, &IncidenceWrapper::incidencePtrChanged, &m_remindersModel, [=](KCalendarCore::Incidence::Ptr incidencePtr) {
-    //     m_remindersModel.setIncidencePtr(incidencePtr);
-    // });
-    // connect(this, &IncidenceWrapper::incidencePtrChanged, &m_attendeesModel, [=](KCalendarCore::Incidence::Ptr incidencePtr) {
-    //     m_attendeesModel.setIncidencePtr(incidencePtr);
-    // });
-    // connect(this, &IncidenceWrapper::incidencePtrChanged, &m_recurrenceExceptionsModel, [=](KCalendarCore::Incidence::Ptr incidencePtr) {
-    //     m_recurrenceExceptionsModel.setIncidencePtr(incidencePtr);
-    // });
-    // connect(this, &IncidenceWrapper::incidencePtrChanged, &m_attachmentsModel, [=](KCalendarCore::Incidence::Ptr incidencePtr) {
-    //     m_attachmentsModel.setIncidencePtr(incidencePtr);
-    // });
-
+//     connect(this, &IncidenceWrapper::incidencePtrChanged, &m_attendeesModel, [=](KCalendarCore::Incidence::Ptr incidencePtr) {
+//         m_attendeesModel.setIncidencePtr(incidencePtr);
+//     });
+//     connect(this, &IncidenceWrapper::incidencePtrChanged, &m_recurrenceExceptionsModel, [=](KCalendarCore::Incidence::Ptr incidencePtr) {
+//         m_recurrenceExceptionsModel.setIncidencePtr(incidencePtr);
+//     });
+//     connect(this, &IncidenceWrapper::incidencePtrChanged, &m_attachmentsModel, [=](KCalendarCore::Incidence::Ptr incidencePtr) {
+//         m_attachmentsModel.setIncidencePtr(incidencePtr);
+//     });
+//     
     // While generally we know of the relationship an incidence has regarding its parent,
     // from the POV of an incidence, we have no idea of its relationship to its children.
     // This is a limitation of KCalendarCore, which only supports one type of relationship
     // type per incidence and throughout the PIM infrastructure it is always the 'parent'
     // relationship that is used.
-
+    
     // We therefore need to rely on the ETMCalendar for this information. Since the ETMCalendar
     // does not provide us with any specific information about the incidences changed when it
     // updates, we unfortunately have to this the coarse way and just update everything when
     // things change.
     connect(CalendarManager::instance(), &CalendarManager::calendarChanged, this, &IncidenceWrapper::resetChildIncidences);
-
+    
     Akonadi::ItemFetchScope scope;
     scope.fetchFullPayload();
     scope.fetchAllAttributes();
     scope.setFetchRelations(true);
     scope.setAncestorRetrieval(Akonadi::ItemFetchScope::Parent);
     setFetchScope(scope);
-
+    
     setNewEvent();
 }
 
@@ -77,7 +74,6 @@ void IncidenceWrapper::notifyDataChanged()
     Q_EMIT durationDisplayStringChanged();
     Q_EMIT allDayChanged();
     Q_EMIT priorityChanged();
-    Q_EMIT remindersModelChanged();
     Q_EMIT organizerChanged();
     // Q_EMIT attendeesModelChanged();
     Q_EMIT recurrenceDataChanged();
@@ -86,6 +82,7 @@ void IncidenceWrapper::notifyDataChanged()
     Q_EMIT todoCompletedChanged();
     Q_EMIT todoCompletionDtChanged();
     Q_EMIT todoPercentCompleteChanged();
+    Q_EMIT googleConferenceUrlChanged();
 }
 
 Akonadi::Item IncidenceWrapper::incidenceItem() const
@@ -98,7 +95,7 @@ void IncidenceWrapper::setIncidenceItem(const Akonadi::Item &incidenceItem)
     if (incidenceItem.hasPayload<KCalendarCore::Incidence::Ptr>()) {
         setItem(incidenceItem);
         setIncidencePtr(incidenceItem.payload<KCalendarCore::Incidence::Ptr>());
-
+        
         Q_EMIT incidenceItemChanged();
         Q_EMIT collectionIdChanged();
     } else {
@@ -114,10 +111,10 @@ KCalendarCore::Incidence::Ptr IncidenceWrapper::incidencePtr() const
 void IncidenceWrapper::setIncidencePtr(const KCalendarCore::Incidence::Ptr incidencePtr)
 {
     m_incidence = incidencePtr;
-
+    
     KCalendarCore::Incidence::Ptr originalIncidence(incidencePtr->clone());
     m_originalIncidence = originalIncidence;
-
+    
     Q_EMIT incidencePtrChanged(incidencePtr);
     Q_EMIT originalIncidencePtrChanged();
     notifyDataChanged();
@@ -255,11 +252,11 @@ void IncidenceWrapper::setIncidenceStart(const QDateTime &incidenceStart, bool r
     // When we receive dates from QML, these are all set to the local system timezone but
     // have the dates and times we want. We need to preserve date and time but set the new
     // QDateTime to have the correct timezone.
-
+    
     // When we set the timeZone property, however, we invariably also set the incidence start and end.
     // This object needs no change. We therefore need to make sure to preserve the entire QDateTime object here.
     auto oldStart = this->incidenceStart();
-
+    
     if (respectTimeZone) {
         m_incidence->setDtStart(incidenceStart);
         auto newTzEnd = incidenceEnd();
@@ -274,11 +271,11 @@ void IncidenceWrapper::setIncidenceStart(const QDateTime &incidenceStart, bool r
         start.setTime(time);
         m_incidence->setDtStart(start);
     }
-
+    
     auto oldStartEndDifference = oldStart.secsTo(incidenceEnd());
     auto newEnd = this->incidenceStart().addSecs(oldStartEndDifference);
     setIncidenceEnd(newEnd);
-
+    
     Q_EMIT incidenceStartChanged();
     Q_EMIT incidenceStartDateDisplayChanged();
     Q_EMIT incidenceStartTimeDisplayChanged();
@@ -290,10 +287,10 @@ void IncidenceWrapper::setIncidenceStartDate(int day, int month, int year)
 {
     QDate date;
     date.setDate(year, month, day);
-
+    
     auto newStart = incidenceStart();
     newStart.setDate(date);
-
+    
     setIncidenceStart(newStart, true);
 }
 
@@ -301,10 +298,10 @@ void IncidenceWrapper::setIncidenceStartTime(int hours, int minutes)
 {
     QTime time;
     time.setHMS(hours, minutes, 0);
-
+    
     auto newStart = incidenceStart();
     newStart.setTime(time);
-
+    
     setIncidenceStart(newStart, true);
 }
 
@@ -342,7 +339,7 @@ void IncidenceWrapper::setIncidenceEnd(const QDateTime &incidenceEnd, bool respe
         end.setDate(date);
         end.setTime(time);
     }
-
+    
     if (m_incidence->type() == KCalendarCore::Incidence::IncidenceType::TypeEvent) {
         KCalendarCore::Event::Ptr event = m_incidence.staticCast<KCalendarCore::Event>();
         event->setDtEnd(end);
@@ -363,10 +360,10 @@ void IncidenceWrapper::setIncidenceEndDate(int day, int month, int year)
 {
     QDate date;
     date.setDate(year, month, day);
-
+    
     auto newEnd = incidenceEnd();
     newEnd.setDate(date);
-
+    
     setIncidenceEnd(newEnd, true);
 }
 
@@ -374,10 +371,10 @@ void IncidenceWrapper::setIncidenceEndTime(int hours, int minutes)
 {
     QTime time;
     time.setHMS(hours, minutes, 0);
-
+    
     auto newEnd = incidenceEnd();
     newEnd.setTime(time);
-
+    
     setIncidenceEnd(newEnd, true);
 }
 
@@ -418,13 +415,13 @@ void IncidenceWrapper::setTimeZone(const QByteArray &timeZone)
         start.setTimeZone(QTimeZone(timeZone));
         setIncidenceStart(start, true);
     }
-
+    
     QDateTime end(incidenceEnd());
     if (end.isValid()) {
         end.setTimeZone(QTimeZone(timeZone));
         setIncidenceEnd(end, true);
     }
-
+    
     Q_EMIT timeZoneChanged();
     Q_EMIT startTimeZoneUTCOffsetMinsChanged();
     Q_EMIT endTimeZoneUTCOffsetMinsChanged();
@@ -445,16 +442,24 @@ KCalendarCore::Duration IncidenceWrapper::duration() const
     return m_incidence->duration();
 }
 
-QString IncidenceWrapper::durationDisplayString() const
+static QString formatSpelloutDuration(const KCalendarCore::Duration &duration, const KFormat &format, bool allDay)
 {
-    const auto dur = duration();
-    if (dur.asSeconds() == 0) {
+    if (duration.asSeconds() == 0) {
         return QString();
     } else {
-        return m_format.formatSpelloutDuration(dur.asSeconds() * 1000);
+        if (allDay) {
+            return format.formatSpelloutDuration(duration.asSeconds() * 1000 + 24 * 60 * 60 * 1000);
+        } else {
+            return format.formatSpelloutDuration(duration.asSeconds() * 1000);
+        }
     }
 }
 
+QString IncidenceWrapper::durationDisplayString() const
+{
+    return formatSpelloutDuration(duration(), m_format, allDay());
+}
+
 bool IncidenceWrapper::allDay() const
 {
     return m_incidence->allDay();
@@ -487,11 +492,11 @@ QVariantMap IncidenceWrapper::recurrenceData()
 {
     QBitArray weekDaysBits = m_incidence->recurrence()->days();
     QVector<bool> weekDaysBools(7);
-
+    
     for (int i = 0; i < weekDaysBits.size(); i++) {
         weekDaysBools[i] = weekDaysBits[i];
     }
-
+    
     QVariantList monthPositions;
     const auto monthPositionsToConvert = m_incidence->recurrence()->monthPositions();
     for (const auto &pos : monthPositionsToConvert) {
@@ -500,7 +505,7 @@ QVariantMap IncidenceWrapper::recurrenceData()
         positionToAdd[QStringLiteral("pos")] = pos.pos();
         monthPositions.append(positionToAdd);
     }
-
+    
     // FYI: yearPositions() just calls monthPositions(), so we're cutting out the middleman
     return QVariantMap{
         {QStringLiteral("weekdays"), QVariant::fromValue(weekDaysBools)},
@@ -526,66 +531,66 @@ void IncidenceWrapper::setRecurrenceDataItem(const QString &key, const QVariant
     if (map.contains(key)) {
         if (key == QStringLiteral("weekdays") && value.canConvert<QJSValue>()) {
             auto jsval = value.value<QJSValue>();
-
+            
             if (!jsval.isArray()) {
                 return;
             }
-
+            
             auto vlist = jsval.toVariant().value<QVariantList>();
             QBitArray days(7);
-
+            
             for (int i = 0; i < vlist.size(); i++) {
                 days[i] = vlist[i].toBool();
             }
-
+            
             KCalendarCore::RecurrenceRule *rrule = m_incidence->recurrence()->defaultRRule();
             QList<KCalendarCore::RecurrenceRule::WDayPos> positions;
-
+            
             for (int i = 0; i < 7; ++i) {
                 if (days.testBit(i)) {
                     KCalendarCore::RecurrenceRule::WDayPos p(0, i + 1);
                     positions.append(p);
                 }
             }
-
+            
             rrule->setByDays(positions);
             m_incidence->recurrence()->updated();
-
+            
         } else if (key == QStringLiteral("duration")) {
             m_incidence->recurrence()->setDuration(value.toInt());
-
+            
         } else if (key == QStringLiteral("frequency")) {
             m_incidence->recurrence()->setFrequency(value.toInt());
-
+            
         } else if ((key == QStringLiteral("startDateTime") || key == QStringLiteral("endDateTime")) && value.toDateTime().isValid()) {
             auto dt = value.toDateTime();
             QDateTime adjustedDt;
             adjustedDt.setTimeZone(incidenceEnd().timeZone());
             adjustedDt.setDate(dt.date());
             adjustedDt.setTime(dt.time());
-
+            
             if (key == QStringLiteral("startDateTime")) {
                 m_incidence->recurrence()->setStartDateTime(adjustedDt, false);
-
+                
             } else if (key == QStringLiteral("endDateTime")) {
                 m_incidence->recurrence()->setEndDateTime(adjustedDt);
             }
-
+            
         } else if (key == QStringLiteral("allDay")) {
             m_incidence->recurrence()->setAllDay(value.toBool());
-
+            
         } else if (key == QStringLiteral("monthDays") && value.canConvert<QList<int>>()) {
             m_incidence->recurrence()->setMonthlyDate(value.value<QList<int>>());
-
+            
         } else if (key == QStringLiteral("yearDays") && value.canConvert<QList<int>>()) {
             m_incidence->recurrence()->setYearlyDay(value.value<QList<int>>());
-
+            
         } else if (key == QStringLiteral("yearDates") && value.canConvert<QList<int>>()) {
             m_incidence->recurrence()->setYearlyDate(value.value<QList<int>>());
-
+            
         } else if (key == QStringLiteral("yearMonths") && value.canConvert<QList<int>>()) {
             m_incidence->recurrence()->setYearlyMonth(value.value<QList<int>>());
-
+            
         } else if (key == QStringLiteral("monthPositions") && value.canConvert<QList<QVariantMap>>()) {
             QList<KCalendarCore::RecurrenceRule::WDayPos> newMonthPositions;
             const auto values = value.value<QList<QVariantMap>>();
@@ -595,19 +600,24 @@ void IncidenceWrapper::setRecurrenceDataItem(const QString &key, const QVariant
                 newPos.setPos(pos[QStringLiteral("pos")].toInt());
                 newMonthPositions.append(newPos);
             }
-
+            
             m_incidence->recurrence()->setMonthlyPos(newMonthPositions);
         }
     }
     Q_EMIT recurrenceDataChanged();
 }
 
+QString IncidenceWrapper::googleConferenceUrl()
+{
+    return m_incidence->customProperty("LIBKGAPI", "EventHangoutLink");
+}
+
 QVariantMap IncidenceWrapper::organizer()
 {
     auto organizerPerson = m_incidence->organizer();
     return QVariantMap{{QStringLiteral("name"), organizerPerson.name()},
-                       {QStringLiteral("email"), organizerPerson.email()},
-                       {QStringLiteral("fullName"), organizerPerson.fullName()}};
+    {QStringLiteral("email"), organizerPerson.email()},
+    {QStringLiteral("fullName"), organizerPerson.fullName()}};
 }
 
 // KCalendarCore::Attendee::List IncidenceWrapper::attendees() const
@@ -615,21 +625,16 @@ QVariantMap IncidenceWrapper::organizer()
 //     return m_incidence->attendees();
 // }
 // 
-// RemindersModel *IncidenceWrapper::remindersModel()
-// {
-//     return &m_remindersModel;
-// }
-// 
 // AttendeesModel *IncidenceWrapper::attendeesModel()
 // {
 //     return &m_attendeesModel;
 // }
-// 
+
 // RecurrenceExceptionsModel *IncidenceWrapper::recurrenceExceptionsModel()
 // {
 //     return &m_recurrenceExceptionsModel;
 // }
-// 
+
 // AttachmentsModel *IncidenceWrapper::attachmentsModel()
 // {
 //     return &m_attachmentsModel;
@@ -640,7 +645,7 @@ bool IncidenceWrapper::todoCompleted()
     if (m_incidence->type() != KCalendarCore::IncidenceBase::TypeTodo) {
         return false;
     }
-
+    
     auto todo = m_incidence.staticCast<KCalendarCore::Todo>();
     return todo->isCompleted();
 }
@@ -650,10 +655,10 @@ void IncidenceWrapper::setTodoCompleted(bool completed)
     if (m_incidence->type() != KCalendarCore::IncidenceBase::TypeTodo) {
         return;
     }
-
+    
     auto todo = m_incidence.staticCast<KCalendarCore::Todo>();
     todo->setCompleted(completed);
-
+    
     Q_EMIT todoCompletionDtChanged();
     Q_EMIT todoPercentCompleteChanged();
     Q_EMIT incidenceIconNameChanged();
@@ -665,7 +670,7 @@ QDateTime IncidenceWrapper::todoCompletionDt()
     if (m_incidence->type() != KCalendarCore::IncidenceBase::TypeTodo) {
         return {};
     }
-
+    
     auto todo = m_incidence.staticCast<KCalendarCore::Todo>();
     return todo->completed();
 }
@@ -675,7 +680,7 @@ int IncidenceWrapper::todoPercentComplete()
     if (m_incidence->type() != KCalendarCore::IncidenceBase::TypeTodo) {
         return 0;
     }
-
+    
     auto todo = m_incidence.staticCast<KCalendarCore::Todo>();
     return todo->percentComplete();
 }
@@ -685,16 +690,16 @@ void IncidenceWrapper::setTodoPercentComplete(int todoPercentComplete)
     if (m_incidence->type() != KCalendarCore::IncidenceBase::TypeTodo) {
         return;
     }
-
+    
     auto todo = m_incidence.staticCast<KCalendarCore::Todo>();
     todo->setPercentComplete(todoPercentComplete);
-
+    
     Q_EMIT todoPercentCompleteChanged();
-
+    
     if (todoPercentComplete < 100 && todoCompleted()) {
         setTodoCompleted(false);
     }
-
+    
     Q_EMIT todoCompletedChanged();
 }
 
@@ -719,14 +724,14 @@ void IncidenceWrapper::setNewEvent()
     start.setSecsSinceEpoch(nearestQuarterHour(QDateTime::currentSecsSinceEpoch()));
     event->setDtStart(start);
     event->setDtEnd(start.addSecs(60 * 60));
-
+    
     KCalendarCore::Alarm::Ptr alarm(new KCalendarCore::Alarm(event.get()));
     alarm->setEnabled(true);
     alarm->setType(KCalendarCore::Alarm::Display);
     alarm->setStartOffset(-1 * 15 * 60); // 15 minutes
-
+    
     event->addAlarm(alarm);
-
+    
     setNewIncidence(event);
 }
 
@@ -757,7 +762,7 @@ void IncidenceWrapper::updateParentIncidence()
     if (!m_incidence) {
         return;
     }
-
+    
     if (!parent().isEmpty() && (!m_parentIncidence || m_parentIncidence->uid() != parent())) {
         m_parentIncidence.reset(new IncidenceWrapper);
         m_parentIncidence->setIncidenceItem(CalendarManager::instance()->incidenceItem(parent()));
@@ -768,20 +773,20 @@ void IncidenceWrapper::updateParentIncidence()
 void IncidenceWrapper::resetChildIncidences()
 {
     cleanupChildIncidences();
-
+    
     if (!m_incidence) {
         return;
     }
-
+    
     const auto incidences = CalendarManager::instance()->childIncidences(uid());
     QVariantList wrappedIncidences;
-
+    
     for (const auto &incidence : incidences) {
         const auto wrappedIncidence = new IncidenceWrapper;
         wrappedIncidence->setIncidenceItem(CalendarManager::instance()->incidenceItem(incidence));
         wrappedIncidences.append(QVariant::fromValue(wrappedIncidence));
     }
-
+    
     m_childIncidences = wrappedIncidences;
     Q_EMIT childIncidencesChanged();
 }
@@ -791,7 +796,7 @@ void IncidenceWrapper::cleanupChildIncidences()
     while (!m_childIncidences.isEmpty()) {
         const auto incidence = m_childIncidences.takeFirst();
         const auto incidencePtr = incidence.value<IncidenceWrapper *>();
-
+        
         delete incidencePtr;
     }
 }
@@ -806,25 +811,25 @@ void IncidenceWrapper::addAlarms(KCalendarCore::Alarm::List alarms)
 void IncidenceWrapper::setRegularRecurrence(IncidenceWrapper::RecurrenceIntervals interval, int freq)
 {
     switch (interval) {
-    case Daily:
-        m_incidence->recurrence()->setDaily(freq);
-        Q_EMIT recurrenceDataChanged();
-        return;
-    case Weekly:
-        m_incidence->recurrence()->setWeekly(freq);
-        Q_EMIT recurrenceDataChanged();
-        return;
-    case Monthly:
-        m_incidence->recurrence()->setMonthly(freq);
-        Q_EMIT recurrenceDataChanged();
-        return;
-    case Yearly:
-        m_incidence->recurrence()->setYearly(freq);
-        Q_EMIT recurrenceDataChanged();
-        return;
-    default:
-        qWarning() << "Unknown interval for recurrence" << interval;
-        return;
+        case Daily:
+            m_incidence->recurrence()->setDaily(freq);
+            Q_EMIT recurrenceDataChanged();
+            return;
+        case Weekly:
+            m_incidence->recurrence()->setWeekly(freq);
+            Q_EMIT recurrenceDataChanged();
+            return;
+        case Monthly:
+            m_incidence->recurrence()->setMonthly(freq);
+            Q_EMIT recurrenceDataChanged();
+            return;
+        case Yearly:
+            m_incidence->recurrence()->setYearly(freq);
+            Q_EMIT recurrenceDataChanged();
+            return;
+        default:
+            qWarning() << "Unknown interval for recurrence" << interval;
+            return;
     }
 }
 
@@ -850,10 +855,17 @@ void IncidenceWrapper::clearRecurrences()
 void IncidenceWrapper::itemChanged(const Akonadi::Item &item)
 {
     if (item.hasPayload<KCalendarCore::Incidence::Ptr>()) {
-        qDebug() << item.payload<KCalendarCore::Incidence::Ptr>()->summary() << item.parentCollection().id();
+       qDebug()<< item.payload<KCalendarCore::Incidence::Ptr>()->summary() << item.parentCollection().id();
         setIncidenceItem(item);
     }
 }
+
+// TODO remove with 22.08, won't be needed anymore
+void IncidenceWrapper::setCollection(const Akonadi::Collection &collection)
+{
+    setCollectionId(collection.id());
+}
+
 #ifndef UNITY_CMAKE_SUPPORT
 Q_DECLARE_METATYPE(KCalendarCore::Incidence::Ptr)
 #endif
